@@ -14,19 +14,19 @@ class WizardStep:
     title: str
     long_description: typing.Optional[str] = None
 
-    def __init__(self, api: API):
+    def __init__(self, api: API.API):
         self.api = api
         self.property_changed_event = Event.Event()
 
     @staticmethod
     def get_ui_description(ui: Declarative.DeclarativeUI) -> Declarative.UIDescription:
-        ...
+        raise NotImplementedError()
 
     def run(self) -> int:
-        ...
+        raise NotImplementedError()
 
     def cancel(self) -> None:
-        ...
+        raise NotImplementedError()
 
 
 class WizardUIHandler(Declarative.Handler):
