@@ -1,6 +1,5 @@
 from . import DoubleGaussian
 from . import GraphicsTools
-from . import MeasureZLP
 from . import SequenceSplitJoin
 from . import AlignMultiSI
 from . import MakeIDPC
@@ -8,3 +7,5 @@ from . import AffineTransformImage
 from . import MakeColorCOM
 from . import AlignSequenceOfMultiDimensionalData
 from . import MultiDimensionalProcessing
+from . import IESquarePlot
+from . import FindLocalMaxima
